<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ElaboracaoCardapio extends \PME\Services\TeknisaCrud {
   protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $connection;
	protected $newCode;
	protected $environment;
	protected $entityManager;
	protected $lista;

    public function __construct($crudService, $parameterBag, $connection, $newCode, $environment, $entityManager, $lista) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->lista = $lista;
	}
	
	/* ------------------------ VERIFICA AS REGRAS QUE UM CARDAPIO PRECISAR TER PARA SER INSERIDO ------------------------*/ 
	public function menuRules(DTO\Request\Row $request, DTO\Response $response) {
		$dataCard = $request->getRow();
		$message = array(
				'isCorrect' => false,
				'message' => ''
			);
		
		try {
			// if ($dataCard['NRCARDPME'] == null) { // Retirado pq n tava verificando caso estivesse no mesmo cardapio.
			$menuExtent = $this->menuExtent($dataCard);
			if (count($menuExtent) > 0) {
				$message['message'] = 'pme::periodoJaCadastrado';
				return $response->addDataSet(new DataSet("message", $message));
			}
			$config = $this->configService($dataCard);
			if (count($config) > 0) {
				$paramPlaneja = $this->getParamPlaneja($dataCard['cdfilcentpme']);
				if ($paramPlaneja[0]['IDCUSPADCAR'] === 'S') {
					$costDefault = $this->costDefault($dataCard);
					if (count($costDefault) == 0 || $costDefault[0]['VRCUSPADTCAR'] === 0) {
						$message['message'] = 'pme::servicoSemCustoPadr';
						return $response->addDataSet(new DataSet("message", $message));
					}
				}
			} else {
				$message['message'] = 'pme::servicoSemConfig';
				return $response->addDataSet(new DataSet("message", $message));
			}
			// }
			$message['isCorrect'] = true;
			return $response->addDataSet(new DataSet("message", $message));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	// Verifica se ja possui dados no banco com a data selecionada.
	private function menuExtent($dataCard) {
		$nrorg = $this->environment->getNrOrgTrab();
		$sql = "SELECT CDFILCENTPME, DTINICARDPME, DTFINCARDPME, CDTIPOCARD FROM CARDAPIOPME, ( SELECT MIN(DTINICARDPME) DTINI, MAX(DTFINCARDPME) DTFIN FROM CARDAPIOPME WHERE CDFILCENTPME =:P_CDFILCENTPME AND CDTIPOCARD =:P_CDTIPOCARD AND DTINICARDPME BETWEEN :P_DTINICARD AND :P_DTFINCARD AND DTFINCARDPME BETWEEN :P_DTINICARD AND :P_DTFINCARD AND NRORG =:NRORG ) INTERVALO WHERE CDFILCENTPME =:P_CDFILCENTPME AND CDTIPOCARD =:P_CDTIPOCARD AND (NRCARDPME <> :P_NRCARDPME OR :P_NRCARDPME IS NULL) AND ( (:P_DTINICARD BETWEEN DTINICARDPME AND DTFINCARDPME OR :P_DTFINCARD BETWEEN DTINICARDPME AND DTFINCARDPME ) OR ( DTINICARDPME BETWEEN DTINI AND DTFIN OR DTFINCARDPME BETWEEN DTINI AND DTFIN ) )";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_CDTIPOCARD"   => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_DTINICARD"    => $dataCard['initialDate'],
            "P_DTFINCARD"    => $dataCard['finalDate'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME'],
            "NRORG"          => $nrorg
        );
      
      return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// Valida se o serviço tem configuração realizada.
	private function configService($dataCard) {
		$sql = "SELECT CDFILCENTPME, CDSERVFILI, NRSEQCNFTCAR, CDTIPOPRAT, IDOPCCNFTCAR, IDIMPCNFTCAR, NRORDCNFTCAR FROM CONFPADTCARD WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDTIPOCARD = :P_CDTIPOCARD AND CDSERVFILI = :P_CDSERVFILI";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_CDTIPOCARD"   => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// Valida se custo parametrizado.
	private function costDefault($dataCard) {
		$nrorg = $this->environment->getNrOrgTrab();
		$sql = "SELECT A.DTREFCUSTCAR, A.VRCUSPADTCAR, A.NRORG FROM (SELECT MAX(DTREFCUSTCAR) AS DTREFCUSTCAR, VRCUSPADTCAR, NRORG FROM CUSTPADTCARD WHERE (CDFILCENTPME =  :P_CDFILCENTPME) AND (CDTIPOCARD   =  :P_CDTIPOCARD)   AND (CDSERVFILI   =  :P_CDSERVFILI)   AND (DTREFCUSTCAR <= :P_DTINIPERIODO) AND (NRORG = :NRORG) GROUP BY VRCUSPADTCAR, NRORG) A";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_CDTIPOCARD"   => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
            "P_DTINIPERIODO" => $dataCard['initialDate'],
            "NRORG"          => $nrorg
        );
       
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* ----------------------------------------------------------------------------*/
	
	/* ------------------------ MONTA OS DADOS DO CARDAPIO ------------------------*/ 
	public function getDataCard(DTO\Request\Row $request, DTO\Response $response) {
		$dataCard = $request->getRow();
		
		try {
			$comens = $this->getComens($dataCard);
			$dishes = $this->getDishes($dataCard);
			$contentPlate = $this->getContentPlate($dataCard);
			$dates = $this->getMenuDate($dataCard);
			$colors = ($dataCard['aggregated'] != null) ? $this->getColors($dataCard) : [];
			
			$allData = array(
				'comens'  => $comens,
				'dishes'  => $dishes,
				'content' => $contentPlate,
				'dates'   => $dates,
				'colors'  => $colors,
			);
			
			$response->addDataSet(new DataSet("allData", $allData));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function getMenuDate($dataCard) {
		$sql = "SELECT DT.DTDATA AS DTCARDIAPME FROM DATA DT WHERE (DT.DTDATA BETWEEN :P_DTINICIAL AND :P_DTFINAL)";
		$paramSelect = array(
            "P_DTINICIAL" => $dataCard['initialDate'],
            "P_DTFINAL"    => $dataCard['finalDate']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getColors($dataCard) {
		$sql = "SELECT PARAMPLANEJA.NRCORNALTE, PARAMPLANEJA.NRCORNUTIL, PARAMPLANEJA.IDCARDIAUTIL, PARAMPLANEJA.IDSOMACOMEST, PARAMGERAL.QTDIABLOQCAR, NVL(PARAMPLANEJA.NRCORCARDESP, 16711680) AS NRCORCARDESP, PARAMGERAL.IDPESQPRATNOME FROM PARAMPLANEJA, PARAMGERAL WHERE PARAMPLANEJA.CDFILIAL = :P_CDFILIAL AND PARAMPLANEJA.CDPARPLANEJA = '1'";
		$paramSelect = array(
            "P_CDFILIAL" => $dataCard['aggregated']['CDFILIAL']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getDishes($dataCard) {
		$sql = "SELECT C.CDFILCENTPME, C.CDSERVFILI, C.NRCARDPME, C.NRSEQCNFCPME, C.CDTIPOPRAT, T.NMTIPOPRAT, C.IDOPCCNFCPME, C.NRORDCNFCPME FROM CONFCARDPME C, TIPOPRAT T WHERE C.CDFILCENTPME =:P_CDFILCENTPME AND C.CDSERVFILI =:P_CDSERVFILI AND C.NRCARDPME =:P_NRCARDPME AND C.CDTIPOPRAT = T.CDTIPOPRAT AND C.NRORG = T.NRORG ORDER BY NRORDCNFCPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getComens($dataCard) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME, IDCARDESPEC, NRORG FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL ORDER BY DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
            "P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
            "P_DTINICIAL"    => $dataCard['initialDate'],
            "P_DTFINAL"      => $dataCard['finalDate']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getContentPlate($dataCard) {
		$sql = "SELECT CARD.CDFILCENTPME, CARD.NRCARDPME, CARD.NRCARDIAPME, :P_CDFILIAL AS CDFILIAL, CARD.CDSERVFILI, 
					  DT.DTDATA AS DTCARDIAPME, C.NRORDCNFCPME, C.NRSEQCNFCPME, C.CDTIPOPRAT, T.NMTIPOPRAT, C.IDOPCCNFCPME, 
					  C.IDIMPCNFCPME, C.NRORDCNFCPME, CARD.NRSEQPRACPME, CARD.NRORDPRACPME, CARD.CDFILAUXPRAT, CARD.CDPRATO, 
					  CARD.NMPRATO, CARD.VRPECONSCPME 
				 FROM DATA DT 
				 LEFT JOIN CONFCARDPME C ON C.CDFILCENTPME = :P_CDFILCENTPME AND C.NRCARDPME = :P_NRCARDPME AND C.CDSERVFILI = :P_CDSERVFILI 
				INNER JOIN TIPOPRAT T ON C.CDTIPOPRAT = T.CDTIPOPRAT 
				 LEFT JOIN (SELECT CC.CDFILCENTPME, CC.NRCARDPME, CC.NRCARDIAPME, :P_CDFILIAL AS CDFILIAL, CD.CDSERVFILI,
				                   CD.DTCARDIAPME, CC.NRSEQPRACPME, CC.NRORDPRACPME, CC.CDFILAUXPRAT, CC.CDPRATO, PT.NMPRATO,
				                   CC.CDTIPOPRAT, CC.VRPECONSCPME 
				              FROM PRATO PT, COMPCARDPME CC, 
				                   (SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDFILIAL, DTCARDIAPME, CDSERVFILI 
				                      FROM CARDIAPME P 
				                     WHERE (P.CDFILCENTPME = :P_CDFILCENTPME) AND (P.NRCARDPME = :P_NRCARDPME) 
				                       AND (P.CDSERVFILI = :P_CDSERVFILI) AND (P.CDFILIAL = P.CDFILCENTPME) 
				                       AND NOT EXISTS(SELECT CDFILIAL FROM CARDIAPME C 
				                                       WHERE (C.CDFILCENTPME = P.CDFILCENTPME) AND (C.NRCARDPME = P.NRCARDPME) 
				                                         AND (C.CDSERVFILI = P.CDSERVFILI) AND (C.CDFILIAL = :P_CDFILIAL) 
				                                         AND (C.DTCARDIAPME = P.DTCARDIAPME) AND (C.IDCARDESPEC = 'S') )
				                     UNION ALL 
				                    SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDFILIAL, DTCARDIAPME, CDSERVFILI 
				                      FROM CARDIAPME 
				                     WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (NRCARDPME = :P_NRCARDPME) 
				                       AND (CDSERVFILI = :P_CDSERVFILI) AND (CDFILIAL = :P_CDFILIAL) AND (IDCARDESPEC = 'S')) CD 
				               WHERE (CC.CDFILCENTPME = CD.CDFILCENTPME) AND (CC.NRCARDPME = CD.NRCARDPME) 
				                 AND (CC.NRCARDIAPME = CD.NRCARDIAPME) AND (PT.CDFILAUXPRAT = CC.CDFILAUXPRAT) 
				                 AND (PT.CDPRATO = CC.CDPRATO) ORDER BY 5, 7) CARD ON DT.DTDATA = CARD.DTCARDIAPME 
				                                                                  AND CARD.CDTIPOPRAT = C.CDTIPOPRAT 
				                                                                  AND CARD.NRSEQPRACPME = C.NRSEQCNFCPME AND CARD.NRORDPRACPME = C.NRORDCNFCPME
				WHERE DT.DTDATA BETWEEN :P_DTINICIAL AND :P_DTFINAL 
				ORDER BY DT.DTDATA, C.NRORDCNFCPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
            "P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
            "P_DTINICIAL"    => $dataCard['initialDate'],
            "P_DTFINAL"      => $dataCard['finalDate']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* ----------------------------------------------------------------------------*/
	
	/* ------------------------ INSERE UM NOVO CARDAPIO E UMA NOVA CONFIGURACAO  ------------------------*/
	public function insertNewCard(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	    	$this->connection->beginTransaction();
	    	$params = $request->getRow();
	    	$cdfilcentpme = $params['cdfilcentpme'];
	      
	    	$inserCard = "INSERT INTO CARDAPIOPME (CDFILCENTPME, NRCARDPME, DSCARDPME, DTINICARDPME, DTFINCARDPME, CDTIPOCARD) VALUES (:CDFILCENTPME, :NRCARDPME, :DSCARDPME, :DTINICARDPME, :DTFINCARDPME, :CDTIPOCARD)";
	    	$nrorg = $this->environment->getNrOrgTrab();
	      
	    	$NRCARDPME = ($params['NRCARDPME'] == null) ? $this->newCode->getCodeByTableName('CARDAPIOPME'.$cdfilcentpme, $nrorg) : $params['NRCARDPME'];
	    	if ($params['NRCARDPME'] == null) {
	    		$paramsTitleMenu = array(
	                "CDFILCENTPME" => $cdfilcentpme,
	                "NRCARDPME"    => $NRCARDPME,
	                "DSCARDPME"    => $params['describeCard'],
	                "DTINICARDPME" => $params['initialDate'],
	                "DTFINCARDPME" => $params['finalDate'],
	                "CDTIPOCARD"   => $params['typeMenu']['CDTIPOCARD']
	            );
            
	            $stmt = $this->connection->prepare($inserCard);
	            $stmt->execute($paramsTitleMenu);
	    	}
	    	
         $this->insertConfigCard($params, $cdfilcentpme, $NRCARDPME); 
         $this->connection->commit();
         $response->addDataSet(new DataSet("NRCARDPME", array($NRCARDPME)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function insertConfigCard($request, $cdfilcentpme, $NRCARDPME) {
		$configCard = "INSERT INTO CONFCARDPME (CDFILCENTPME, NRCARDPME, CDSERVFILI, NRSEQCNFCPME, CDTIPOPRAT, IDOPCCNFCPME, IDIMPCNFCPME, NRORDCNFCPME) SELECT CDFILCENTPME, :P_NRCARDPME, CDSERVFILI, NRSEQCNFTCAR, CDTIPOPRAT, IDOPCCNFTCAR, IDIMPCNFTCAR, NRORDCNFTCAR FROM CONFPADTCARD WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDTIPOCARD = :P_CDTIPOCARD AND CDSERVFILI = :P_CDSERVFILI";
		
		$paramsConfigCard = array(
            "P_CDFILCENTPME" => $cdfilcentpme,
            "P_NRCARDPME"    => $NRCARDPME,
            "P_CDSERVFILI"   => $request['service']['CDSERVFILI'],
            "P_CDTIPOCARD"   => $request['typeMenu']['CDTIPOCARD']
        );
        $stmt = $this->connection->prepare($configCard);
        $stmt->execute($paramsConfigCard);
	}
	/*------------------------------------------------------------------------*/
	
	/* ------------------------ INSERE DADOS DO PRATO SEGUINDO AS REGRAS ------------------------*/
	public function insertInCardia($row) {
		$row['filterData']['CDFILIALINSERT'] = $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : NULL;
		$cardInDay = $this->checkCardInDay($row);
		
		if (count($cardInDay) == 0) {
			$this->newCardia($row);
			$cardInDay = $this->checkCardInDay($row);
			
			if($row['filterData']['aggregated'] == null && $row['IDCARDESPEC'] == 'N') $this->insertAggregated($row);
		}
		
		return $cardInDay;
	}
	
	// VERIFICA SE POSSUI CARDAPIO NO DIA SELECIONADO NA CELULA.
	public function checkCardInDay($row) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTCARDIAPME = :P_DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
            "P_NRCARDPME"    => $row['filterData']['NRCARDPME'],
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_CDFILIAL"     => $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	public function newCardia($row) {
		try {
			$newCardia = "INSERT INTO CARDIAPME (CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME, IDCARDESPEC) VALUES (:CDFILCENTPME, :NRCARDPME, :NRCARDIAPME, :CDSERVFILI, :CDFILIAL, :DTCARDIAPME, :QTCOMESTCPME, :QTCLIESTCPME, :QTFUNESTCPME, :QTDOAESTCPME, :QTCOMEFECPME, :QTCLIEFECPME, :QTFUNEFECPME, :QTDOAEFECPME, :IDCARDESPEC)";
	        $this->connection->beginTransaction();
	        
	        $holiday = $this->isHoliday($row);
	        $holidayInCard = $this->isHolidayInCard($row);
	        $paramPlaneja = $this->getParamPlaneja($row['filterData']['cdfilcentpme']);
	        
	        $diners = $this->getDinersCard($row);
	        
	        // VERIFICA SE O CARDAPIO POSSUI FERIADO
	        if(count($holiday) != 0 && count($holidayInCard) != 0 && $paramPlaneja[0]['IDCARDIAUTIL'] != 'N') {
	        	$row['WEEKDAY'] = 7;
	        }
	        
	        $qtcomen = $this->getComenOfDay($diners[0], $row['WEEKDAY']);
	        
	        $nrorg = $this->environment->getNrOrgTrab();
	        $newNrcardiapme  = $this->newCode->getCodeByTableName('CARDIAPME'.$row['filterData']['cdfilcentpme'].$row['filterData']['NRCARDPME'], $nrorg);
			
	        $insertNewCardia = array(
	            "CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
	            "NRCARDPME"    => $row['filterData']['NRCARDPME'],
	            "NRCARDIAPME"  => $newNrcardiapme,
	            "CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
	            "CDFILIAL"     => $row['filterData']['CDFILIALINSERT'] != NULL ? $row['filterData']['CDFILIALINSERT'] : $row['filterData']['cdfilcentpme'],
	            "DTCARDIAPME"  => $row['DTCARDIAPME'],
	            "QTCOMESTCPME" => $qtcomen['QTCOMESTCPME'], 
	            "QTCLIESTCPME" => $qtcomen['QTCLIESTCPME'],
	            "QTFUNESTCPME" => $qtcomen['QTFUNESTCPME'], 
	            "QTDOAESTCPME" => $qtcomen['QTDOAESTCPME'],
	            "QTCOMEFECPME" => 0, 
	            "QTCLIEFECPME" => 0, 
	            "QTFUNEFECPME" => 0, 
	            "QTDOAEFECPME" => 0,
	            "IDCARDESPEC"  => $row['filterData']['aggregated'] == null ? 'N' : 'S'
	        );
	        
	        $stmt = $this->connection->prepare($newCardia);
        	$stmt->execute($insertNewCardia);
        	$this->connection->commit();
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	// PEGA AS COMENSAIS DA CENTRAL (AGREGADA OU CARDAPIO ESPECIFICO)
	public function getDinersCard($row) {
		$sql = "SELECT A.QTCOMPMESEG, A.QTCOMPMETER, A.QTCOMPMEQUA, A.QTCOMPMEQUI, A.QTCOMPMESEX, A.QTCOMPMESAB, A.QTCOMPMEDOM, A.QTCOMPMEFER, A.QTCLIPMESEG, A.QTCLIPMETER, A.QTCLIPMEQUA, A.QTCLIPMEQUI, A.QTCLIPMESEX, A.QTCLIPMESAB, A.QTCLIPMEDOM, A.QTCLIPMEFER, A.QTFUNPMESEG, A.QTFUNPMETER, A.QTFUNPMEQUA, A.QTFUNPMEQUI, A.QTFUNPMESEX, A.QTFUNPMESAB, A.QTFUNPMEDOM, A.QTFUNPMEFER, A.QTDOAPMESEG, A.QTDOAPMETER, A.QTDOAPMEQUA, A.QTDOAPMEQUI, A.QTDOAPMESEX, A.QTDOAPMESAB, A.QTDOAPMEDOM, A.QTDOAPMEFER FROM ( SELECT SUM(QTCOMPMESEG) AS QTCOMPMESEG, SUM(QTCOMPMETER) AS QTCOMPMETER, SUM(QTCOMPMEQUA) AS QTCOMPMEQUA, SUM(QTCOMPMEQUI) AS QTCOMPMEQUI, SUM(QTCOMPMESEX) AS QTCOMPMESEX, SUM(QTCOMPMESAB) AS QTCOMPMESAB, SUM(QTCOMPMEDOM) AS QTCOMPMEDOM, SUM(QTCOMPMEFER) AS QTCOMPMEFER, SUM(QTCLIPMESEG) AS QTCLIPMESEG, SUM(QTCLIPMETER) AS QTCLIPMETER, SUM(QTCLIPMEQUA) AS QTCLIPMEQUA, SUM(QTCLIPMEQUI) AS QTCLIPMEQUI, SUM(QTCLIPMESEX) AS QTCLIPMESEX, SUM(QTCLIPMESAB) AS QTCLIPMESAB, SUM(QTCLIPMEDOM) AS QTCLIPMEDOM, SUM(QTCLIPMEFER) AS QTCLIPMEFER, SUM(QTFUNPMESEG) AS QTFUNPMESEG, SUM(QTFUNPMETER) AS QTFUNPMETER, SUM(QTFUNPMEQUA) AS QTFUNPMEQUA, SUM(QTFUNPMEQUI) AS QTFUNPMEQUI, SUM(QTFUNPMESEX) AS QTFUNPMESEX, SUM(QTFUNPMESAB) AS QTFUNPMESAB, SUM(QTFUNPMEDOM) AS QTFUNPMEDOM, SUM(QTFUNPMEFER) AS QTFUNPMEFER, SUM(QTDOAPMESEG) AS QTDOAPMESEG, SUM(QTDOAPMETER) AS QTDOAPMETER, SUM(QTDOAPMEQUA) AS QTDOAPMEQUA, SUM(QTDOAPMEQUI) AS QTDOAPMEQUI, SUM(QTDOAPMESEX) AS QTDOAPMESEX, SUM(QTDOAPMESAB) AS QTDOAPMESAB, SUM(QTDOAPMEDOM) AS QTDOAPMEDOM, SUM(QTDOAPMEFER) AS QTDOAPMEFER FROM COMESERVTCAR C, FILITIPOCARD F WHERE (C.CDFILCENTPME = :P_CDFILCENTPME) AND (C.CDTIPOCARD   = :P_CDTIPOCARD)   AND (C.CDSERVFILI   = :P_CDSERVFILI)   AND ((C.CDFILIAL     = :P_CDFILIAL OR :P_CDFILIAL IS NULL))     AND (F.CDFILCENTPME = C.CDFILCENTPME)  AND (F.CDTIPOCARD   = C.CDTIPOCARD)    AND (F.CDFILIAL     = C.CDFILIAL)      AND (F.IDTCARATFILI = 'S')) A";
		
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
            "P_CDFILIAL"     => $row['filterData']['CDFILIALINSERT'] != null ? $row['filterData']['CDFILIALINSERT'] : null,
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_CDTIPOCARD"   => $row['filterData']['typeMenu']['CDTIPOCARD']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// VERIFICA SE NO DIA QUE ESTÁ SENDO INSERINDO O PRATO É UM FERIADO (PRA CENTRAL OU AGREGADA ESPECÍFICA)
	private function isHoliday($row) {
		$sql = "SELECT DTFERIFILI, DSFERIFILI FROM FERIFILI WHERE CDFILIAL = :P_CDFILIAL AND DTFERIFILI = :P_DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILIAL" => $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// VERIFICA SE NO DIA QUE ESTÁ SENDO INSERINDO O PRATO É UM FERIADO NO CARDÁPIO (PRA CENTRAL OU AGREGADA ESPECÍFICA)
	private function isHolidayInCard($row) {
		$sql = "SELECT DTFERICARD, DSFERICARD FROM FERICARD WHERE CDFILIAL = :P_CDFILIAL AND DTFERICARD = :P_DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILIAL" => $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// USE DAY OF THE WEEK, WITH SUNDAY AS 0 AND SATURDAY AS 6.
	public function getComenOfDay($diners, $day) {
		switch($day) {
			case 0: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEDOM'], "QTCLIESTCPME" => $diners['QTCLIPMEDOM'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEDOM'], "QTDOAESTCPME" => $diners['QTDOAPMEDOM']		
	        	);
	        	break;
        	case 1: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMESEG'], "QTCLIESTCPME" => $diners['QTCLIPMESEG'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMESEG'], "QTDOAESTCPME" => $diners['QTDOAPMESEG']		
	        	);
	        	break;
        	case 2: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMETER'], "QTCLIESTCPME" => $diners['QTCLIPMETER'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMETER'], "QTDOAESTCPME" => $diners['QTDOAPMETER']		
	        	);
	        	break;
        	case 3: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEQUA'], "QTCLIESTCPME" => $diners['QTCLIPMEQUA'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEQUA'], "QTDOAESTCPME" => $diners['QTDOAPMEQUA']		
	        	);
	        	break;
        	case 4: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEQUI'], "QTCLIESTCPME" => $diners['QTCLIPMEQUI'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEQUI'], "QTDOAESTCPME" => $diners['QTDOAPMEQUI']		
	        	);
	        	break;
        	case 5: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMESEX'], "QTCLIESTCPME" => $diners['QTCLIPMESEX'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMESEX'], "QTDOAESTCPME" => $diners['QTDOAPMESEX']		
	        	);
	        	break;
        	case 6: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMESAB'], "QTCLIESTCPME" => $diners['QTCLIPMESAB'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMESAB'], "QTDOAESTCPME" => $diners['QTDOAPMESAB']		
	        	);
	        	break;
        	case 7: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEFER'], "QTCLIESTCPME" => $diners['QTCLIPMEFER'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEFER'], "QTDOAESTCPME" => $diners['QTDOAPMEFER']		
	        	);
	        	break;
		}
	}
	
	private function insertAggregated($row) {
		$aggregates = $this->getAggregated($row);
		foreach($aggregates as $aggregated) {
			$hasUnity = $this->UnityHaveCard($aggregated['CDFILIAL'], $row);
			if(count($hasUnity) == 0) {
				$row['filterData']['CDFILIALINSERT'] = $aggregated['CDFILIAL'];
				$this->newCardia($row);
			}
		}
	}
	
	/* BUSCA AS AGREGADAS QUE POSSUEM O TIPO DE CARDÁPIO ATIVO */
	private function getAggregated($row) {
		$sql = "SELECT FILITIPOCARD.CDFILIAL FROM FILITIPOCARD, SERVTIPOCARD, SERVICO, PARAMPLANEJA WHERE (FILITIPOCARD.CDFILCENTPME = :P_CDFILCENTPME) AND (FILITIPOCARD.CDTIPOCARD = :P_CDTIPOCARD) AND (SERVTIPOCARD.CDFILCENTPME = FILITIPOCARD.CDFILCENTPME) AND (SERVTIPOCARD.CDTIPOCARD = FILITIPOCARD.CDTIPOCARD) AND (SERVTIPOCARD.CDSERVFILI = :P_CDSERVFILI) AND (SERVICO.CDFILIAL = FILITIPOCARD.CDFILIAL) AND (SERVICO.CDSERVFILI = SERVTIPOCARD.CDSERVFILI) AND (FILITIPOCARD.CDFILIAL = PARAMPLANEJA.CDFILIAL) AND (FILITIPOCARD.IDTCARATFILI = 'S')";
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_CDTIPOCARD"   => $row['filterData']['typeMenu']['CDTIPOCARD']
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* VERIFICA SE TEM CARDÁPIO PARA ESTA UNIDADE */
	private function UnityHaveCard($cdfilial, $row) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTCARDIAPME = :P_DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
            "P_NRCARDPME"    => $row['filterData']['NRCARDPME'],
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME'],
            "P_CDFILIAL"     => $cdfilial
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* ------------------------------------------------------------------------------------------------ */
	
	/* ------------------------ CHAMA A PROCEDURE ATUALIZA_COMENSAL ------------------------*/
	
	public function callUpdateComen(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		try {
			$nrorg = $this->environment->getNrOrgTrab();
			$nrlista = $this->lista->geraLista($row['aggregated']['CDFILIAL'], $nrorg);
			$this->procedureUpdateComen($row, $nrlista);
			$response->addDataSet(new DataSet("updated", array(true)));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function procedureUpdateComen($row, $lista) {
		$connection = $this->entityManager->getConnection();
    	$procedureCons = new StoredProcedure($connection, 'ATUALIZA_COMENSAL');
    	
		$procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $row['cdfilcentpme'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_NRCARDPME',    Param::PARAM_INPUT, $row['NRCARDPME'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_NRLISTA',      Param::PARAM_INPUT, $lista,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $row['service']['CDSERVFILI'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, $row['typeMenu']['CDTIPOCARD'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_DTINI',        Param::PARAM_INPUT, $row['DATAPROCE'], Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_DTFIN',        Param::PARAM_INPUT, $row['finalDate'],   Param::PARAM_TYPE_STR));
    	
    	$procedureCons->execute();
	}
	
	/* ------------------------------------------------------------------------------------------------ */
	
	/* FUNÇÃO PARA PEGAR OS DADOS DA PARAMPLANEJA (PODE ADICIONAR MAIS COLUNAS FUTURAMENTE) */
	private function getParamPlaneja($cdfilial) {
		$sql = "SELECT IDCARDIAUTIL, IDCUSPADCAR, IDPERCCOME FROM PARAMPLANEJA WHERE CDFILIAL = :P_CDFILIAL AND CDPARPLANEJA = '1'";
		$paramSelect = array(
            "P_CDFILIAL" => $cdfilial
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	public function hasConfigCard(DTO\Request\Row $request, DTO\Response $response) {
		$sql = "SELECT * FROM CONFCARDPME WHERE NRCARDPME = :NRCARDPME AND CDSERVFILI = :CDSERVFILI AND CDFILCENTPME = :CDFILCENTPME";
		$hasData = false;
		$row = $request->getRow();
		
		$paramSelect = array(
            "NRCARDPME"    => $row['NRCARDPME'],
            "CDSERVFILI"   => $row['service']['CDSERVFILI'],
            "CDFILCENTPME" => $row['cdfilcentpme']
        );
        
        $paramPlaneja = $this->getParamPlaneja($row['cdfilcentpme']);
        $data = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
        
        $params = array(
        	'hasData' => (count($data) > 0),
        	'IDPERCCOME' => $paramPlaneja[0]['IDPERCCOME']
        );
        
        $response->addDataSet(new DataSet("hasConfigCard", $params));
	}
	
	/* ------------------------------------------------------------------------------------------------ */
}